import typing

import requests


class TrelloApi:
    """
    Base class to interact with Trello API.
    Ideally this should not be used directly
    """

    base_url: str = "https://api.trello.com/1"
    service_endpoint: typing.Optional[str] = None
    url: str
    params: typing.Dict[str, typing.Any] = {}
    headers: typing.Dict[str, typing.Any] = {"Accept": "application/json"}

    def __init__(self, api_key, api_token):
        self.params["key"] = api_key
        self.params["token"] = api_token
        self.url = f"{self.base_url.strip('/')}/{self.service_endpoint.strip('/')}"


class Boards(TrelloApi):
    base_url = "https://api.trello.com/1/members/me"
    service_endpoint = "/boards"

    def list(self):
        params = self.params.copy()
        params["fields"] = "name"
        req = requests.get(self.url, params=params, headers=self.headers)
        return req.json()


class Lists(TrelloApi):
    service_endpoint = "/boards/{board_id}/lists"
    board_id = None

    def list(self):
        req = requests.get(
            self.url.format(board_id=self.board_id),
            params=self.params,
            headers=self.headers,
        )
        return req.json()


class Cards(TrelloApi):
    service_endpoint = "/cards"

    def create(self, card):
        query = self.params.copy()
        query.update(card.__dict__)

        response = requests.post(self.url, headers=self.headers, params=query)
        return response.json()



import pandas


class Card:
    """
    Simple class used to map Trello's Card.
    """

    name = None
    desc = None
    idList = None  # noqa Since this is mapping a Trello field, it does not follow PEP8

    def __init__(self, **kwargs):
        for key, value in kwargs.items():
            if hasattr(self, key):
                setattr(self, key, value)

    def __repr__(self) -> str:
        return f"{self.name}/{self.desc}/{self.idList}"

    def __eq__(self, other_card: object) -> bool:
        comparable_attrs = list(
            filter(lambda m: not (m.startswith("__")), Card.__dict__)
        )
        comparisson = [
            getattr(self, attr) == getattr(other_card, attr)
            for attr in comparable_attrs
        ]
        return all(comparisson)


def card_from_dict(attrs):
    """
    Creates a new Card object from a dict as input.
    This is used to create Card from both CLI and CSV inputs.
    The keys used will only be the keys that have exactly the same name
    as the attributes in Card class.
    """

    acceptable_params = list(filter(lambda m: not (m.startswith("__")), Card.__dict__))
    new_card_params = {param: attrs.get(param, None) for param in acceptable_params}
    return Card(**new_card_params)


def cards_from_csv(cards_csv):
    """
    Returns a list of Card objects for a CSV input.
    The CSV column headers must coincide with Card's attributes in name, not in order.
    """
    df = pandas.read_csv(cards_csv)
    return [Card(**row.to_dict()) for _idx, row in df.iterrows()]



import argparse

from . import api, model

__version__ = "0.7.0"


def board_list_handler(api_credentials, _args):
    return api.Boards(**api_credentials).list()


def list_handler(api_credentials, args):
    list_ids = api.Lists(**api_credentials)
    list_ids.board_id = args.board_id
    return list_ids.list()


def card_create_handler(api_credentials, args):
    if args.cards_csv:
        cards_list = model.cards_from_csv(args.cards_csv)
    else:
        cards_list = [model.card_from_dict(args.__dict__)]

    return [api.Cards(**api_credentials).create(card) for card in cards_list]


def main():
    parser = argparse.ArgumentParser(description="Interacts with Trello Restful API")
    parser.add_argument("--api_key")
    parser.add_argument("--api_token")

    subparsers = parser.add_subparsers(help="service help")

    boards = subparsers.add_parser("boards")
    board_actions = boards.add_subparsers()

    board_list = board_actions.add_parser("list")
    board_list.set_defaults(func=board_list_handler)

    lists = subparsers.add_parser("lists")
    lists.add_argument("--board_id", dest="board_id")
    lists.set_defaults(func=list_handler)

    cards = subparsers.add_parser("cards")
    card_actions = cards.add_subparsers()

    card_create = card_actions.add_parser("create")
    card_csv = card_create.add_argument_group("create from csv")
    card_csv.add_argument(
        "-f", "--from-csv", dest="cards_csv", type=argparse.FileType("r")
    )

    card_model = card_create.add_argument_group("create from cli")
    card_model.add_argument("--name", dest="name", type=str)
    card_model.add_argument("--desc", dest="desc", type=str)
    card_model.add_argument("--idList", dest="idList", type=str)

    card_create.set_defaults(func=card_create_handler)
    args = parser.parse_args(
        "cards create --name test --desc test --idList abc123".split()
    )

    args = parser.parse_args()

    api_credentials = {"api_key": args.api_key, "api_token": args.api_token}

    try:
        print(args.func(api_credentials, args))
    except AttributeError:
        parser.print_help()
